@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2009, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	
+ *	$Id: misc.c,v 1.26.2.5 2010-03-31 14:12:02 ssinger Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -31,11 +31,6 @@
 #endif
 #include <stdarg.h>
 
-#include "libpq-fe.h"
-#include "c.h"
-
-#include "confoptions.h"
-
 #include "slon.h"
 
 
@@ -64,8 +59,8 @@ extern char *Syslog_facility;	/* openlog() parameters */
 extern char *Syslog_ident;
 
 static void write_syslog(int level, const char *line);
-#else
 
+#else    /* HAVE_SYSLOG */
 #define Use_syslog 0
 #endif   /* HAVE_SYSLOG */
 
@@ -81,64 +76,76 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 	static char *outbuf = NULL;
 	static int	outsize = -1;
 	int			off;
+	int  len;
 	char	   *level_c = NULL;
 
 	char		time_buf[128];   /* Buffer to hold timestamp */
 	char        ps_buf[20];      /* Buffer to hold PID */
 	time_t		stamp_time = time(NULL);
 
+	
 #ifdef HAVE_SYSLOG
 	int			syslog_level = LOG_ERR;
 #endif
 	if (level > slon_log_level)
 		return;
-
-	switch (level)
-	{
-		case SLON_DEBUG4:
-			level_c = "DEBUG4";
-			break;
-		case SLON_DEBUG3:
-			level_c = "DEBUG3";
-			break;
-		case SLON_DEBUG2:
-			level_c = "DEBUG2";
-			break;
-		case SLON_DEBUG1:
-			level_c = "DEBUG1";
+    switch (level)
+    {
+        case SLON_DEBUG4:
+            level_c = "DEBUG4";
 #ifdef HAVE_SYSLOG
-			syslog_level = LOG_DEBUG;
+            syslog_level = LOG_DEBUG;
 #endif
-			break;
-		case SLON_INFO:
-			level_c = "INFO";
+            break;
+        case SLON_DEBUG3:
+            level_c = "DEBUG3";
 #ifdef HAVE_SYSLOG
-			syslog_level = LOG_INFO;
+            syslog_level = LOG_DEBUG;
 #endif
-			break;
-		case SLON_CONFIG:
-			level_c = "CONFIG";
-			break;
-		case SLON_WARN:
-			level_c = "WARN";
+            break;
+        case SLON_DEBUG2:
+            level_c = "DEBUG2";
 #ifdef HAVE_SYSLOG
-			syslog_level = LOG_WARNING;
+            syslog_level = LOG_DEBUG;
 #endif
-			break;
-		case SLON_ERROR:
-			level_c = "ERROR";
+            break;
+        case SLON_DEBUG1:
+            level_c = "DEBUG1";
 #ifdef HAVE_SYSLOG
-			syslog_level = LOG_ERR;
-#endif
-			break;
-		case SLON_FATAL:
-			level_c = "FATAL";
+            syslog_level = LOG_DEBUG;
+#endif                                                                          
+            break;                                                              
+        case SLON_INFO:                                                         
+            level_c = "INFO";                                                   
+#ifdef HAVE_SYSLOG                                                              
+            syslog_level = LOG_INFO;                                            
+#endif                                                                          
+            break;                                                              
+        case SLON_CONFIG:                                                       
+            level_c = "CONFIG";                                                 
 #ifdef HAVE_SYSLOG
-			syslog_level = LOG_ERR;
+            syslog_level = LOG_WARNING;
 #endif
-
-			break;
-	}
+            break;                                                              
+        case SLON_WARN:                                                         
+            level_c = "WARN";                                                   
+#ifdef HAVE_SYSLOG
+            syslog_level = LOG_WARNING;
+#endif
+            break;                                                              
+        case SLON_ERROR:                                                        
+            level_c = "ERROR";                                                  
+#ifdef HAVE_SYSLOG                                                              
+            syslog_level = LOG_ERR;                                             
+#endif                                                                          
+            break;                                                              
+        case SLON_FATAL:                                                        
+            level_c = "FATAL";                                                  
+#ifdef HAVE_SYSLOG                                                              
+            syslog_level = LOG_ERR;                                             
+#endif                                                                          
+            break;                                                              
+    }
 
 	va_start(ap, fmt);
 
@@ -146,7 +153,7 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 	if (outbuf == NULL)
 	{
 		outsize = 8192;
-		outbuf = malloc(outsize);
+		outbuf = malloc((size_t) outsize);
 		if (outbuf == NULL)
 		{
 			perror("slon_log: malloc()");
@@ -154,7 +161,7 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 			slon_retry();
 		}
 	}
-	outbuf[0] = 0;
+	outbuf[0] = (char) 0;
 
 	if (logtimestamp == true && (Use_syslog != 1)
 #ifdef WIN32
@@ -162,26 +169,31 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 #endif
 		)
 	{
-		strftime(time_buf, sizeof(time_buf), log_timestamp_format, localtime(&stamp_time));
+		len = (int) strftime(time_buf, sizeof(time_buf), log_timestamp_format, localtime(&stamp_time));
+		if (len == 0 && time_buf[0] != '\0') {
+			perror("slon_log: problem with strftime()");
+			slon_retry();
+		}
 	} else {
 		time_buf[0] = (char) 0;
 	}
+
 	if (logpid == true)
 	{
 		sprintf(ps_buf, "[%d] ", slon_pid);
 	} else {
 		ps_buf[0] = (char) 0;
 	}
+
 	sprintf(outbuf, "%s%s%-6.6s ", time_buf, ps_buf, level_c);
 
-	off = strlen(outbuf);
-
+	off = (int) strlen(outbuf);
 	va_list apcopy;
-	va_copy(apcopy, ap);
-	while (vsnprintf(&outbuf[off], outsize - off - 1, fmt, apcopy) >= outsize - off - 1)
+	va_copy(apcopy,ap);
+	while (vsnprintf(&outbuf[off], (size_t) (outsize - off), fmt, apcopy) >= outsize - off - 1)
 	{
 		outsize *= 2;
-		outbuf = realloc(outbuf, outsize);
+		outbuf = realloc(outbuf, (size_t) outsize);
 		if (outbuf == NULL)
 		{
 			perror("slon_log: realloc()");
@@ -203,8 +215,8 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 	if (win32_isservice)
 		win32_eventlog(level, outbuf);
 #endif
-	fwrite(outbuf, strlen(outbuf), 1, stdout);
-	fflush(stdout);
+	(void) fwrite(outbuf, strlen(outbuf), 1, stdout);
+	(void) fflush(stdout);
 	pthread_mutex_unlock(&log_mutex);
 
 	va_end(ap);
@@ -248,7 +260,7 @@ slon_scanint64(char *str, int64 * result)
 		if (strcmp(ptr, "9223372036854775808") == 0)
 		{
 			*result = -INT64CONST(0x7fffffffffffffff) - 1;
-			return true;
+			return (int) true;
 		}
 #endif
 	}
@@ -290,7 +302,7 @@ write_syslog(int level, const char *line)
 	static unsigned long seq = 0;
 	static int	syslog_fac = LOG_LOCAL0;
 
-	int			len = strlen(line);
+	int			len = (int) strlen(line);
 
 	if (Use_syslog == 0)
 		return;
@@ -348,7 +360,7 @@ write_syslog(int level, const char *line)
 			if (strchr(buf, '\n') != NULL)
 				*strchr(buf, '\n') = '\0';
 
-			buflen = strlen(buf);
+			buflen = (int) strlen(buf);
 
 			if (buflen <= 0)
 				return;
