@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2009, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	
+ *	$Id: slonik.c,v 1.91.2.10 2010-03-30 15:55:41 ssinger Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -14,38 +14,28 @@
 #ifndef WIN32
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <errno.h>
 #include <time.h>
 #else
-#include <windows.h>
 #define sleep(x) Sleep(x*1000)
-
+#define vsnprintf _vsnprintf
 #endif
 
-#include<string.h>
-#include <errno.h>
-#include <stdarg.h>
-
-
-#include "types.h"
+#include "postgres.h"
 #include "libpq-fe.h"
+#include "port.h"
+
 #include "slonik.h"
-
-
-#ifdef MSVC
-#include "config_msvc.h"
-#else
 #include "config.h"
-#endif
 #include "../parsestatements/scanner.h"
 extern int STMTS[MAXSTATEMENTS];
 
-#define MAXPGPATH 256
-
 /*
  * Global data
  */
@@ -117,7 +107,14 @@ main(int argc, const char *argv[])
 	/*
 	 * We need to find a share directory like PostgreSQL. 
 	 */
-	strcpy(share_path, PGSHARE);
+	if (strlen(PGSHARE) > 0)
+	{
+		strcpy(share_path, PGSHARE);
+	}
+	else
+	{
+		get_share_path(myfull_path, share_path);
+	}
 
 	if (optind < argc)
 	{
@@ -940,15 +937,6 @@ script_check_stmts(SlonikScript * script, SlonikStmt * hdr)
 					SlonikStmt_ddl_script *stmt =
 					(SlonikStmt_ddl_script *) hdr;
 
-					if ((stmt->only_on_node > -1) && (stmt->ev_origin > -1)
-							&& (stmt->ev_origin != stmt->only_on_node)) {
-						printf ("If ONLY ON NODE is given, "
-								"EVENT ORIGIN must be the same node");
-						errors++;
-					}
-					if (stmt->only_on_node > -1)
-						stmt->ev_origin = stmt->only_on_node;
-
 					if (stmt->ev_origin < 0)
 					{
 						printf("%s:%d: Error: require EVENT NODE\n", 
@@ -1545,13 +1533,15 @@ static void
 script_commit_all(SlonikStmt * stmt, SlonikScript * script)
 {
 	SlonikAdmInfo *adminfo;
+	int			error = 0;
 
 	for (adminfo = script->adminfo_list;
 		 adminfo; adminfo = adminfo->next)
 	{
 		if (adminfo->dbconn != NULL && adminfo->have_xact)
 		{
-		  db_commit_xact(stmt, adminfo);
+			if (db_commit_xact(stmt, adminfo) < 0)
+				error = 1;
 		}
 		else
 		{
@@ -1760,8 +1750,9 @@ static int
 load_slony_base(SlonikStmt * stmt, int no_id)
 {
 	SlonikAdmInfo *adminfo;
-	int			rc;
+	PGconn	   *dbconn;
 	SlonDString query;
+	int			rc;
 
 	int			use_major = 0;
 	int			use_minor = 0;
@@ -1769,6 +1760,8 @@ load_slony_base(SlonikStmt * stmt, int no_id)
 	if ((adminfo = get_active_adminfo(stmt, no_id)) == NULL)
 		return -1;
 
+	dbconn = adminfo->dbconn;
+
 	rc = db_check_namespace(stmt, adminfo, stmt->script->clustername);
 	if (rc > 0)
 	{
@@ -1810,16 +1803,8 @@ load_slony_base(SlonikStmt * stmt, int no_id)
 	{
 		use_major = 8;
 		use_minor = 4;   
-	}		
-	else if ((adminfo->pg_version >= 90000) && (adminfo->pg_version < 0100)) /* 9.0 */
-	{
-		/**
-		 * 9.0 and so far just like 8.4
-		 **/
-		use_major=8;
-		use_minor=4;
 	}
-	else	/* 9.1 or higher */
+	else	/* above 8.4 ??? */
 	{
 		use_major = 8;
 		use_minor = 4;
@@ -1867,7 +1852,7 @@ static int
 load_slony_functions(SlonikStmt * stmt, int no_id)
 {
 	SlonikAdmInfo *adminfo;
-	
+	PGconn	   *dbconn;
 
 	int			use_major = 0;
 	int			use_minor = 0;
@@ -1875,6 +1860,7 @@ load_slony_functions(SlonikStmt * stmt, int no_id)
 	if ((adminfo = get_active_adminfo(stmt, no_id)) == NULL)
 		return -1;
 
+	dbconn = adminfo->dbconn;
 
         /* determine what schema version we should load */
 
@@ -1896,23 +1882,15 @@ load_slony_functions(SlonikStmt * stmt, int no_id)
 		use_major = 8;
 		use_minor = 4;
 	}
-	else if ((adminfo->pg_version >= 90000) && (adminfo->pg_version < 90100)) /* 9.0 */
-	{
-		/**
-		 * 9.0 is so far just like 8.4
-		 */
-		use_major = 8;
-		use_minor = 4;
-	}
-	else    /* above 9.1 */
-	{
-		use_major = 8;
-		use_minor = 4;
-		printf("%s:%d: Possible unsupported PostgreSQL "
-			   "version (%d) %d.%d, defaulting to 8.4 support\n",
+        else    /* above 8.4 */
+        {
+                use_major = 8;
+                use_minor = 4;
+                printf("%s:%d: Possible unsupported PostgreSQL "
+                       "version (%d) %d.%d, defaulting to 8.4 support\n",
 		       stmt->stmt_filename, stmt->stmt_lno, adminfo->pg_version,
 		       (adminfo->pg_version/10000), ((adminfo->pg_version%10000)/100));
-	}
+        }
 
 	/* Load schema, DB version specific */
 	db_notice_silent = true;
@@ -2392,8 +2370,8 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 
 	failnode_node *nodeinfo;
 	failnode_set *setinfo;
-	char	   *failsetbuf;
-	char	   *failnodebuf;
+	char	   *configbuf;
+
 	PGresult   *res1;
 	PGresult   *res2;
 	PGresult   *res3;
@@ -2457,21 +2435,24 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	/*
 	 * Allocate and initialize memory to hold some config info
 	 */
-	failsetbuf = malloc( sizeof(failnode_set) * num_sets);
-	failnodebuf = malloc( sizeof(failnode_node) * (num_nodes
-												   +num_sets*num_nodes));
-	memset(failsetbuf,0,sizeof(failnode_set) * num_sets);
-	memset(failnodebuf,0,sizeof(failnode_node) * (num_nodes 
-												  + (num_sets * num_nodes) ));
-	
-	nodeinfo = (failnode_node *) failnodebuf;
-	setinfo = (failnode_set *) failsetbuf;
+	configbuf = malloc(
+					   MAXALIGN(sizeof(failnode_node) * num_nodes) +
+					   MAXALIGN(sizeof(failnode_set) * num_sets) +
+					   sizeof(failnode_node *) * num_nodes * num_sets);
+	memset(configbuf, 0,
+		   MAXALIGN(sizeof(failnode_node) * num_nodes) +
+		   MAXALIGN(sizeof(failnode_set) * num_sets) +
+		   sizeof(failnode_node *) * num_nodes * num_sets);
 
+	nodeinfo = (failnode_node *) configbuf;
+	setinfo = (failnode_set *) (configbuf +
+								MAXALIGN(sizeof(failnode_node) * num_nodes));
 	for (i = 0; i < num_sets; i++)
 	{
 		setinfo[i].subscribers = (failnode_node **)
-			(failnodebuf+ sizeof(failnode_node) * 
-			 (num_nodes + (i*num_nodes))); 
+			(configbuf + MAXALIGN(sizeof(failnode_node) * num_nodes) +
+			 MAXALIGN(sizeof(failnode_set) * num_sets) +
+			 sizeof(failnode_node *) * num_nodes * i);
 	}
 
 	/*
@@ -2486,8 +2467,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		if (nodeinfo[i].adminfo == NULL)
 		{
 			PQclear(res1);
-			free(failnodebuf);
-			free(failsetbuf);
+			free(configbuf);
 			dstring_free(&query);
 			return -1;
 		}
@@ -2507,8 +2487,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		{
 			PQclear(res1);
 			PQclear(res2);
-			free(failnodebuf);
-			free(failsetbuf);
+			free(configbuf);
 			dstring_free(&query);
 			return -1;
 		}
@@ -2550,8 +2529,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		res3 = db_exec_select((SlonikStmt *) stmt, adminfo1, &query);
 		if (res3 == NULL)
 		{
-			free(failnodebuf);
-			free(failsetbuf);
+			free(configbuf);
 			dstring_free(&query);
 			return -1;
 		}
@@ -2575,8 +2553,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 			{
 				printf("node %d not found - inconsistent configuration\n",
 					   sub_receiver);
-				free(failnodebuf);
-				free(failsetbuf);
+				free(configbuf);
 				PQclear(res3);
 				PQclear(res2);
 				dstring_free(&query);
@@ -2597,8 +2574,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 				 stmt->no_id, stmt->backup_node);
 	if (db_exec_command((SlonikStmt *) stmt, adminfo1, &query) < 0)
 	{
-		free(failnodebuf);
-		free(failsetbuf);
+		free(configbuf);
 		dstring_free(&query);
 		return -1;
 	}
@@ -2609,8 +2585,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 
 		if (db_exec_command((SlonikStmt *) stmt, nodeinfo[i].adminfo, &query) < 0)
 		{
-			free(failnodebuf);
-			free(failsetbuf);
+			free(configbuf);
 			dstring_free(&query);
 			return -1;
 		}
@@ -2623,8 +2598,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	{
 		if (db_commit_xact((SlonikStmt *) stmt, nodeinfo[i].adminfo) < 0)
 		{
-			free(failnodebuf);
-			free(failsetbuf);
+			free(configbuf);
 			dstring_free(&query);
 			return -1;
 		}
@@ -2659,8 +2633,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 			res1 = db_exec_select((SlonikStmt *) stmt, nodeinfo[i].adminfo, &query);
 			if (res1 == NULL)
 			{
-				free(failnodebuf);
-				free(failsetbuf);
+				free(configbuf);
 				dstring_free(&query);
 				return -1;
 			}
@@ -2673,8 +2646,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 			PQclear(res1);
 			if (db_rollback_xact((SlonikStmt *) stmt, nodeinfo[i].adminfo) < 0)
 			{
-				free(failnodebuf);
-				free(failsetbuf);
+				free(configbuf);
 				dstring_free(&query);
 				return -1;
 			}
@@ -2737,8 +2709,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 								  adminfo1, &query);
 			if (res1 == NULL)
 			{
-				free(failnodebuf);
-				free(failsetbuf);
+				free(configbuf);
 				dstring_free(&query);
 				return -1;
 			}
@@ -2766,9 +2737,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 								  setinfo[i].subscribers[j]->adminfo, &query);
 			if (res1 == NULL)
 			{
-				free(failsetbuf);
-				free(failnodebuf);
-				
+				free(configbuf);
 				dstring_free(&query);
 				return -1;
 			}
@@ -2814,10 +2783,12 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	for (i = 0; i < num_sets; i++)
 	{
 		int			use_node;
+		SlonikAdmInfo *use_adminfo;
 
 		if (setinfo[i].num_directsub <= 1)
 		{
 			use_node = stmt->backup_node;
+			use_adminfo = adminfo1;
 		}
 		else if (setinfo[i].max_node == NULL)
 		{
@@ -2825,6 +2796,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 				   setinfo[i].set_id);
 			rc = -1;
 			use_node = stmt->backup_node;
+			use_adminfo = adminfo1;
 		}
 		else
 		{
@@ -2833,6 +2805,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 				   setinfo[i].set_id,
 				   setinfo[i].max_seqno);
 			use_node = setinfo[i].max_node->no_id;
+			use_adminfo = setinfo[i].max_node->adminfo;
 
 			setinfo[i].max_node->num_sets++;
 		}
@@ -2891,8 +2864,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 			rc = -1;
 	}
 
-	free(failsetbuf);
-	free(failnodebuf);
+	free(configbuf);
 	dstring_free(&query);
 	return rc;
 }
@@ -3455,73 +3427,17 @@ int
 slonik_subscribe_set(SlonikStmt_subscribe_set * stmt)
 {
 	SlonikAdmInfo *adminfo1;
-	SlonikAdmInfo *adminfo2;
 	SlonDString query;
-	PGresult    *res1;
-	int reshape=0;
-	int origin_id;
 
 	adminfo1 = get_active_adminfo((SlonikStmt *) stmt, stmt->sub_provider);
 	if (adminfo1 == NULL)
 		return -1;
 
-	
+	if (db_begin_xact((SlonikStmt *) stmt, adminfo1) < 0)
+		return -1;
+
 	dstring_init(&query);
 
-	/**
-	 * If the receiver is already subscribed to
-	 * the set through a different provider
-	 * slonik will need to tell the receiver
-	 * about this change directly.
-	 */
-
-	slon_mkquery(&query,"select count(*) FROM \"_%s\".sl_subscribe " \
-				 "where sub_set=%d AND sub_receiver=%d " \
-				 " and sub_active=true and sub_provider<>%d",
-				 stmt->hdr.script->clustername,
-				 stmt->sub_setid,stmt->sub_receiver,
-				 stmt->sub_provider);
-	
-	res1 = db_exec_select((SlonikStmt*) stmt,adminfo1,&query);
-	if(res1 == NULL) {
-		dstring_free(&query);
-		return -1;
-	}
-	if (strtol(PQgetvalue(res1, 0, 0), NULL, 10) > 0)
-	{
-		reshape=1;
-	}
-	PQclear(res1);
-	dstring_reset(&query);
-
-	slon_mkquery(&query,
-				 "select set_origin from \"_%s\".sl_set where" \
-				 " set_id=%d",stmt->hdr.script->clustername,
-				 stmt->sub_setid);
-	res1 = db_exec_select((SlonikStmt*)stmt,adminfo1,&query);
-	if(res1==NULL || PQntuples(res1)==0 ) 
-	{
-		PQclear(res1);
-		dstring_free(&query);
-		printf("%s:%d Error: set %d not found. \n",
-			   stmt->hdr.stmt_filename, stmt->hdr.stmt_lno,
-			   stmt->sub_setid);
-		return -1;
-
-	}
-	origin_id = atoi(PQgetvalue(res1,0,0));
-	if(origin_id <= 0 ) 
-	{
-		PQclear(res1);
-		dstring_free(&query);
-		return -1;
-		
-	}
-	PQclear(res1);
-	dstring_reset(&query);
-	adminfo2 = get_active_adminfo((SlonikStmt *) stmt, origin_id);
-	if (db_begin_xact((SlonikStmt *) stmt, adminfo2) < 0)
-		return -1;
 	slon_mkquery(&query,
 				 "select \"_%s\".subscribeSet(%d, %d, %d, '%s', '%s'); ",
 				 stmt->hdr.script->clustername,
@@ -3529,34 +3445,13 @@ slonik_subscribe_set(SlonikStmt_subscribe_set * stmt)
 				 stmt->sub_receiver,
 				 (stmt->sub_forward) ? "t" : "f",
 				 (stmt->omit_copy) ? "t" : "f");
-	if (db_exec_evcommand((SlonikStmt *) stmt, adminfo2, &query) < 0)
+	if (db_exec_evcommand((SlonikStmt *) stmt, adminfo1, &query) < 0)
 	{
 		dstring_free(&query);
 		return -1;
 	}
-	dstring_reset(&query);
-	if(reshape)
-	{
-		adminfo2 = get_active_adminfo((SlonikStmt *) stmt, stmt->sub_receiver);
-		if(adminfo2 == NULL) 
-		{
-			printf("can not find conninfo for receiver node %d\n",
-				   stmt->sub_receiver);
-			return -1;
-		}
-		slon_mkquery(&query,
-					 "select \"_%s\".reshapeSubscription(%d,%d,%d);",
-					 stmt->hdr.script->clustername,
-					 stmt->sub_setid,
-					 stmt->sub_provider,
-					 stmt->sub_receiver);	
-		if (db_exec_command((SlonikStmt *) stmt, adminfo2, &query) < 0)
-		{
-			printf("error reshaping subscriber\n");		
-		}
-		
-		dstring_free(&query);
-	}
+
+	dstring_free(&query);
 	return 0;
 }
 
@@ -3749,9 +3644,9 @@ slonik_ddl_script(SlonikStmt_ddl_script * stmt)
 
 #define PARMCOUNT 1  
 
-	const char *params[PARMCOUNT];
-	int paramlens[PARMCOUNT];
-	int paramfmts[PARMCOUNT];
+        const char *params[PARMCOUNT];
+        int paramlens[PARMCOUNT];
+        int paramfmts[PARMCOUNT];
 
 	adminfo1 = get_active_adminfo((SlonikStmt *) stmt, stmt->ev_origin);
 	if (adminfo1 == NULL)
