@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2009, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	
+ *	$Id: slonik.h,v 1.34.2.2 2009-08-17 17:09:59 devrim Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -24,6 +24,8 @@ typedef struct SlonikStmt_store_node_s SlonikStmt_store_node;
 typedef struct SlonikStmt_drop_node_s SlonikStmt_drop_node;
 typedef struct SlonikStmt_failed_node_s SlonikStmt_failed_node;
 typedef struct SlonikStmt_uninstall_node_s SlonikStmt_uninstall_node;
+typedef struct SlonikStmt_clone_prepare_s SlonikStmt_clone_prepare;
+typedef struct SlonikStmt_clone_finish_s SlonikStmt_clone_finish;
 typedef struct SlonikStmt_store_path_s SlonikStmt_store_path;
 typedef struct SlonikStmt_drop_path_s SlonikStmt_drop_path;
 typedef struct SlonikStmt_store_listen_s SlonikStmt_store_listen;
@@ -37,9 +39,6 @@ typedef struct SlonikStmt_set_drop_table_s SlonikStmt_set_drop_table;
 typedef struct SlonikStmt_set_drop_sequence_s SlonikStmt_set_drop_sequence;
 typedef struct SlonikStmt_set_move_table_s SlonikStmt_set_move_table;
 typedef struct SlonikStmt_set_move_sequence_s SlonikStmt_set_move_sequence;
-typedef struct SlonikStmt_table_add_key_s SlonikStmt_table_add_key;
-typedef struct SlonikStmt_store_trigger_s SlonikStmt_store_trigger;
-typedef struct SlonikStmt_drop_trigger_s SlonikStmt_drop_trigger;
 typedef struct SlonikStmt_subscribe_set_s SlonikStmt_subscribe_set;
 typedef struct SlonikStmt_unsubscribe_set_s SlonikStmt_unsubscribe_set;
 typedef struct SlonikStmt_lock_set_s SlonikStmt_lock_set;
@@ -55,43 +54,42 @@ typedef struct SlonikStmt_sleep_s SlonikStmt_sleep;
 typedef enum
 {
 	STMT_TRY = 1,
+	STMT_CLONE_FINISH,
+	STMT_CLONE_PREPARE,
 	STMT_CREATE_SET,
 	STMT_DDL_SCRIPT,
 	STMT_DROP_LISTEN,
 	STMT_DROP_NODE,
 	STMT_DROP_PATH,
 	STMT_DROP_SET,
-	STMT_DROP_TRIGGER,
 	STMT_ECHO,
+	STMT_ERROR,
 	STMT_EXIT,
 	STMT_FAILED_NODE,
 	STMT_INIT_CLUSTER,
 	STMT_LOCK_SET,
 	STMT_MERGE_SET,
 	STMT_MOVE_SET,
-	STMT_RESTART_NODE,
 	STMT_REPAIR_CONFIG,
+	STMT_RESTART_NODE,
 	STMT_SET_ADD_SEQUENCE,
 	STMT_SET_ADD_TABLE,
 	STMT_SET_DROP_SEQUENCE,
 	STMT_SET_DROP_TABLE,
 	STMT_SET_MOVE_SEQUENCE,
 	STMT_SET_MOVE_TABLE,
+	STMT_SLEEP,
 	STMT_STORE_LISTEN,
 	STMT_STORE_NODE,
 	STMT_STORE_PATH,
-	STMT_STORE_TRIGGER,
 	STMT_SUBSCRIBE_SET,
-	STMT_TABLE_ADD_KEY,
+	STMT_SWITCH_LOG,
+	STMT_SYNC,
 	STMT_UNINSTALL_NODE,
 	STMT_UNLOCK_SET,
 	STMT_UNSUBSCRIBE_SET,
 	STMT_UPDATE_FUNCTIONS,
-	STMT_WAIT_EVENT,
-	STMT_SWITCH_LOG,
-	STMT_ERROR,
-	STMT_SYNC,
-	STMT_SLEEP
+	STMT_WAIT_EVENT
 }	Slonik_stmttype;
 
 struct SlonikScript_s
@@ -182,7 +180,6 @@ struct SlonikStmt_store_node_s
 	SlonikStmt	hdr;
 	int			no_id;
 	char	   *no_comment;
-	int			no_spool;
 	int			ev_origin;
 };
 
@@ -210,6 +207,23 @@ struct SlonikStmt_uninstall_node_s
 };
 
 
+struct SlonikStmt_clone_prepare_s
+{
+	SlonikStmt	hdr;
+	int			no_id;
+	int			no_provider;
+	char	   *no_comment;
+};
+
+
+struct SlonikStmt_clone_finish_s
+{
+	SlonikStmt	hdr;
+	int			no_id;
+	int			no_provider;
+};
+
+
 struct SlonikStmt_store_path_s
 {
 	SlonikStmt	hdr;
@@ -279,7 +293,6 @@ struct SlonikStmt_set_add_table_s
 	int			set_id;
 	int			set_origin;
 	int			tab_id;
-	int			use_serial;
 	char	   *use_key;
 	char	   *tab_fqname;
 	char	   *tab_comment;
@@ -331,32 +344,6 @@ struct SlonikStmt_set_move_sequence_s
 };
 
 
-struct SlonikStmt_table_add_key_s
-{
-	SlonikStmt	hdr;
-	int			no_id;
-	char	   *tab_fqname;
-};
-
-
-struct SlonikStmt_store_trigger_s
-{
-	SlonikStmt	hdr;
-	int			trig_tabid;
-	char	   *trig_tgname;
-	int			ev_origin;
-};
-
-
-struct SlonikStmt_drop_trigger_s
-{
-	SlonikStmt	hdr;
-	int			trig_tabid;
-	char	   *trig_tgname;
-	int			ev_origin;
-};
-
-
 struct SlonikStmt_subscribe_set_s
 {
 	SlonikStmt	hdr;
@@ -364,6 +351,7 @@ struct SlonikStmt_subscribe_set_s
 	int			sub_provider;
 	int			sub_receiver;
 	int			sub_forward;
+	int			omit_copy;
 };
 
 
@@ -545,6 +533,8 @@ extern int	slonik_store_node(SlonikStmt_store_node * stmt);
 extern int	slonik_drop_node(SlonikStmt_drop_node * stmt);
 extern int	slonik_failed_node(SlonikStmt_failed_node * stmt);
 extern int	slonik_uninstall_node(SlonikStmt_uninstall_node * stmt);
+extern int	slonik_clone_prepare(SlonikStmt_clone_prepare * stmt);
+extern int	slonik_clone_finish(SlonikStmt_clone_finish * stmt);
 extern int	slonik_store_path(SlonikStmt_store_path * stmt);
 extern int	slonik_drop_path(SlonikStmt_drop_path * stmt);
 extern int	slonik_store_listen(SlonikStmt_store_listen * stmt);
@@ -558,9 +548,6 @@ extern int	slonik_set_drop_table(SlonikStmt_set_drop_table * stmt);
 extern int	slonik_set_drop_sequence(SlonikStmt_set_drop_sequence * stmt);
 extern int	slonik_set_move_table(SlonikStmt_set_move_table * stmt);
 extern int	slonik_set_move_sequence(SlonikStmt_set_move_sequence * stmt);
-extern int	slonik_table_add_key(SlonikStmt_table_add_key * stmt);
-extern int	slonik_store_trigger(SlonikStmt_store_trigger * stmt);
-extern int	slonik_drop_trigger(SlonikStmt_drop_trigger * stmt);
 extern int	slonik_subscribe_set(SlonikStmt_subscribe_set * stmt);
 extern int	slonik_unsubscribe_set(SlonikStmt_unsubscribe_set * stmt);
 extern int	slonik_lock_set(SlonikStmt_lock_set * stmt);
@@ -590,17 +577,21 @@ void		db_notice_recv(void *arg, const char *msg);
 int			db_connect(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
 int			db_disconnect(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
 
-int db_exec_command(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
+int			db_exec_command(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
 				SlonDString * query);
-int db_exec_evcommand(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
-				  SlonDString * query);
-PGresult *db_exec_select(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
-			   SlonDString * query);
-int db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
-int db_check_namespace(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
-				   char *clustername);
-int db_check_requirements(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
-					  char *clustername);
+int			db_exec_evcommand(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
+				SlonDString * query);
+int			db_exec_evcommand_p(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
+				SlonDString * query, int nParams, const Oid *paramTypes, 
+				const char *const *paramValues, const int *paramLengths, 
+				const int *paramFormats, int resultFormat);
+PGresult   *db_exec_select(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
+				SlonDString * query);
+int			db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
+int			db_check_namespace(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
+				char *clustername);
+int			db_check_requirements(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
+				char *clustername);
 int			db_get_nodeid(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
 int			db_begin_xact(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
 int			db_commit_xact(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
