@@ -1,5 +1,5 @@
 #!/bin/sh
-# 
+# $Id: run_test.sh,v 1.26.2.5 2010-03-11 21:40:08 cbbrowne Exp $
 
 pgbindir=${PGBINDIR:-"/usr/local/pgsql/bin"}
 numerrors=0
@@ -38,8 +38,11 @@ if [ -z "$2" ]; then
   esac
 fi
 
+TESTSTARTTIME=`date +"%Y-%m-%d %H:%M:%S %Z"`
+
+
 if [ ! -x "$pgbindir/psql" ]; then
-  echo "please set the PGBINDIR envvar to the directory containing psql, createdb, ..."
+  echo "please set the PGBINDIR environment variable to the directory containing psql, createdb, ..."
   exit 1;
 fi
 
@@ -51,10 +54,15 @@ echo "----------------------------------------------------"
 
 #load settings
 
+echo "Loading default settings"
 . settings.ik
+echo "Loading test-specific settings"
 . $testname/settings.ik
+echo "Loading support functions"
 . support_funcs.sh
 
+echo "Test by ${SLONYTESTER} to be summarized in ${SLONYTESTFILE}"
+
 trap '
 	echo ""
 	echo "**** user abort"
@@ -64,17 +72,17 @@ trap '
 
 stop_processes()
 {
-	alias=1
+	node=1
 	while : ; do
-	  eval workerpid=\$worker${alias}_pid
+	  eval workerpid=\$worker${node}_pid
 	  if [ ! -z $workerpid ]; then
-	    echo "**** killing worker $alias"
+	    echo "**** killing worker $node"
 	    kill -15 $workerpid
 	  fi
-	  if [ ${alias} -ge ${WORKERS} ]; then
+	  if [ ${node} -ge ${WORKERS} ]; then
 	    break;
 	  else
-	    alias=`expr ${alias} + 1`
+	    node=`expr ${node} + 1`
 	  fi
 	done
 	stop_poll
@@ -105,58 +113,56 @@ stop_poll()
 
 stop_slons()
 {
-	alias=1
+	node=1
         while : ; do
-          eval slonpid=\$slon${alias}_pid
-	  if [ ! -z $slonpid ]; then
-            echo "**** killing slon node $alias"
-            kill -15 $slonpid
-	  fi
-          if [ ${alias} -ge ${NUMNODES} ]; then
+	  SLON_BUILD=$PGBINDIR SLON_CONF=${mktmp}/slon-conf.${node} $SLTOOLDIR/start_slon.sh stop
+          if [ ${node} -ge ${NUMNODES} ]; then
             break;
           else
-            alias=`expr ${alias} + 1`
+            node=`expr ${node} + 1`
           fi
         done
 }
 
 
 init_preamble() {
-	alias=1
+	node=1
 	while : ; do
-	  eval cluster=\$CLUSTER${alias}
+	  eval cluster=\$CLUSTER${node}
 	  if [ -n "${cluster}" ]; then
-	    echo "CLUSTER NAME = ${cluster};" > $mktmp/slonik.script
-	    if [ ${alias} -ge ${NUMCLUSTERS} ]; then
+	    echo "CLUSTER NAME = ${cluster};" > $mktmp/slonik.preamble
+	    if [ ${node} -ge ${NUMCLUSTERS} ]; then
 	      break;
 	    else
-	      alias=expr ${alias} + 1
+	      node=expr ${node} + 1
 	    fi
 	  else
 	    break;
 	  fi
 	done
 
-	alias=1
+	node=1
 
 	while : ; do
-	  eval db=\$DB${alias}
-	  eval host=\$HOST${alias}
-	  eval user=\$USER${alias}
-	  eval port=\$PORT${alias}
+	  eval db=\$DB${node}
+	  eval host=\$HOST${node}
+	  eval user=\$USER${node}
+	  eval port=\$PORT${node}
+	  echo "define node${node} ${node};" >> $mktmp/slonik.preamble
 	
 	  if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
 	    conninfo="dbname=${db} host=${host} user=${user} port=${port}"
-	    echo "NODE ${alias} ADMIN CONNINFO = '${conninfo}';" >> $mktmp/slonik.script
-	    if [ ${alias} -ge ${NUMNODES} ]; then
+	    echo "NODE ${node} ADMIN CONNINFO = '${conninfo}';" >> $mktmp/slonik.preamble
+	    if [ ${node} -ge ${NUMNODES} ]; then
 	      break;
 	    else
-	      alias=`expr ${alias} + 1`
+	      node=`expr ${node} + 1`
 	    fi   
 	  else
 	    break;
 	  fi
 	done
+	echo "include <${mktmp}/slonik.preamble>;" > $mktmp/slonik.script
 }
 
 
@@ -169,26 +175,21 @@ store_node()
   eval oport=\$PORT${originnode}
 
   if [ -n "${odb}" -a "${ohost}" -a "${ouser}" -a "${oport}" ]; then
-    alias=1
+    node=1
     while : ; do
-      eval db=\$DB${alias}
-      eval host=\$HOST${alias}
-      eval user=\$USER${alias}
-      eval port=\$PORT${alias}
-      eval logship=\$LOGSHIP${alias}
+      eval db=\$DB${node}
+      eval host=\$HOST${node}
+      eval user=\$USER${node}
+      eval port=\$PORT${node}
 
       if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
-        if [ ${alias} -ne ${originnode} ]; then
-          if [ "x${logship}" == "xtrue" ]; then    # Don't bother generating nodes used for log shipping
-            status "Node ${alias} is a log shipping node - no need for STORE NODE"
-          else
-            echo "STORE NODE (id=${alias}, comment='node ${alias}');" >> $mktmp/slonik.script
-         fi
+        if [ ${node} -ne ${originnode} ]; then
+            echo "STORE NODE (id=@node${node}, comment='node ${node}', event node=${originnode});" >> $mktmp/slonik.script
         fi
-        if [ ${alias} -ge ${NUMNODES} ]; then
+        if [ ${node} -ge ${NUMNODES} ]; then
           break;
         else
-          alias=$((${alias} + 1))
+          node=$((${node} + 1))
         fi
       else
         break;
@@ -207,7 +208,6 @@ store_path()
     eval host=\$HOST${i}
     eval user=\$USER${i}
     eval port=\$PORT${i}
-    eval logship=\$LOGSHIP${i}
 
     if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
       j=1
@@ -215,18 +215,12 @@ store_path()
         if [ ${i} -ne ${j} ]; then
           eval bdb=\$DB${j}
           eval bhost=\$HOST${j}
-          eval buser=\$USER${j}
+          eval buser=\$WEAKUSER${j}
           eval bport=\$PORT${j}
-          eval blogship=\$LOGSHIP${j}
           if [ -n "${bdb}" -a "${bhost}" -a "${buser}" -a "${bport}" ]; then
-            if [[ "x${logship}" == "xtrue" || "x${blogship}" == "xtrue" ]]; then
-                # log shipping node - no paths need exist that involve this node
-                status "log shipping between nodes(${i}/${j}) - ls(${logship}/${blogship}) - omit STORE PATH"
-            else
-	    echo "STORE PATH (SERVER=${i}, CLIENT=${j}, CONNINFO='dbname=${db} host=${host} user=${user} port=${port}');" >> $mktmp/slonik.script
-            fi
+	    echo "STORE PATH (SERVER=@node${i}, CLIENT=@node${j}, CONNINFO='dbname=${db} host=${host} user=${buser} port=${port}');" >> $mktmp/slonik.script
           else
-            err 3 "No conninfo"
+            err 3 "No conninfo - bdb=${bdb} bhost=${bhost} user=${buser} bport=${bport}"
           fi
         fi
         if [ ${j} -ge ${NUMNODES} ]; then
@@ -252,6 +246,7 @@ init_origin_rdbms()
         eval db=\$DB${originnode}
         eval host=\$HOST${originnode}
         eval user=\$USER${originnode}
+        eval weakuser=\$WEAKUSER${originnode}
 	eval pgbindir=\$PGBINDIR${originnode}
 	eval port=\$PORT${originnode}
 
@@ -259,7 +254,7 @@ init_origin_rdbms()
 	  status "creating origin DB: $user -h $host -U $user -p $port $db"
   	  $pgbindir/createdb -O $user -h $host -U $user -p $port --encoding $ENCODING -T template0 $db 1> ${mktmp}/createdb.${originnode} 2> ${mktmp}/createdb.${originnode}
 	  if [ $? -ne 0 ]; then	   
-	    err 3 "An error occured trying to $pgbindir/createdb -O $user -h $host -U $user -p $port --encoding $ENCODING -T template0 $db, ${mktmp}/createdb.${originnode} for details"
+	    err 3 "An error occurred trying to $pgbindir/createdb -O $user -h $host -U $user -p $port --encoding $ENCODING -T template0 $db, ${mktmp}/createdb.${originnode} for details"
 	  fi
 	else
 	  err 3 "No db '${db}' or host '${host}' or user '${user}' or port '${port}' specified"
@@ -268,6 +263,9 @@ init_origin_rdbms()
         $pgbindir/createlang -h $host -U $user -p $port plpgsql $db
 	status "loading origin DB with $testname/init_schema.sql"
 	$pgbindir/psql -h $host -p $port $db $user < $testname/init_schema.sql 1> ${mktmp}/init_schema.sql.${originnode} 2>${mktmp}/init_schema.sql.${originnode}
+	status "setting up user ${weakuser} to have weak access to data"
+	$SHELL ${testname}/gen_weak_user.sh ${weakuser} > ${mktmp}/grant_weak_access.sql
+                 $pgbindir/psql -h $host -p $port -d $db -U $user < ${mktmp}/grant_weak_access.sql > ${mktmp}/genweakuser.sql.${originnode} 2> ${mktmp}/genweakuser.sql.${originnode}
 	status "done"
 }
 
@@ -277,32 +275,34 @@ create_subscribers()
         eval odb=\$DB${originnode}
         eval ohost=\$HOST${originnode}
         eval ouser=\$USER${originnode}
+        eval oweakuser=\$WEAKUSER${originnode}
 	eval opgbindir=\$PGBINDIR${originnode}
 	eval oport=\$PORT${originnode}
 
         if [ -n "${odb}" -a "${ohost}" -a "${ouser}" ]; then
-          alias=1
+          node=1
           while : ; do
-            eval db=\$DB${alias}
-            eval host=\$HOST${alias}
-            eval user=\$USER${alias}
-	    eval pgbindir=\$PGBINDIR${alias}
-	    eval port=\$PORT${alias}
+            eval db=\$DB${node}
+            eval host=\$HOST${node}
+            eval user=\$USER${node}
+            eval weakuser=\$WEAKUSER${node}
+	    eval pgbindir=\$PGBINDIR${node}
+	    eval port=\$PORT${node}
 
             if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
-              if [ ${alias} -ne ${originnode} ]; then
-		status "creating subscriber ${alias} DB: $user -h $host -U $user -p $port $db"
-	        $pgbindir/createdb -O $user -h $host -U $user -p $port --encoding $ENCODING -T template0 $db 1> ${mktmp}/createdb.${alias} 2> ${mktmp}/createdb.${alias}
+              if [ ${node} -ne ${originnode} ]; then
+		status "creating subscriber ${node} DB: $user -h $host -U $user -p $port $db"
+	        $pgbindir/createdb -O $user -h $host -U $user -p $port --encoding $ENCODING -T template0 $db 1> ${mktmp}/createdb.${node} 2> ${mktmp}/createdb.${node}
 		status "add plpgsql to subscriber"
 		$pgbindir/createlang -h $host -U $user -p $port plpgsql $db
-		status "loading subscriber ${alias} DB from $odb"
-	        $opgbindir/pg_dump -s  -h $ohost -U $ouser -p $oport $odb | $pgbindir/psql -h $host -p $port $db $user 1> ${mktmp}/init_schema.sql.${alias} 2> ${mktmp}/init_schema.sql.${alias}
+		status "loading subscriber ${node} DB from $odb"
+	        $opgbindir/pg_dump -h $ohost -U $ouser -p $oport $odb | $pgbindir/psql -h $host -p $port $db $user 1> ${mktmp}/init_schema.sql.${node} 2> ${mktmp}/init_schema.sql.${node}
 		status "done"
               fi
-              if [ ${alias} -ge ${NUMNODES} ]; then
+              if [ ${node} -ge ${NUMNODES} ]; then
                 break;
               else
-                alias=$((${alias} + 1))
+                node=$((${node} + 1))
               fi   
             else
               break;
@@ -314,25 +314,62 @@ create_subscribers()
 	fi
 }
 
+generate_weak_slony_grants ()
+{
+  node=1
+
+  ROTBLS="sl_action_seq sl_config_lock sl_confirm sl_event
+  sl_event_seq sl_listen sl_local_node_id sl_log_1 sl_log_2
+  sl_log_status sl_node  sl_path sl_registry
+  sl_seqlastvalue sl_seqlog sl_sequence sl_set sl_setsync
+  sl_status sl_subscribe sl_table"
+
+  RWTBLS="sl_nodelock sl_nodelock_nl_conncnt_seq"
+
+  while : ; do
+    eval db=\$DB${node}
+    eval host=\$HOST${node}
+    eval user=\$USER${node}
+    eval weakuser=\$WEAKUSER${node}
+    eval pgbindir=\$PGBINDIR${node}
+    eval port=\$PORT${node}
+
+    if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
+      $pgbindir/psql -h $host -p $port -U $user -d $db -c "grant usage on schema \"_${CLUSTER1}\" to ${weakuser};" > /dev/null 2> /dev/null
+      for table in `echo $ROTBLS`; do
+        $pgbindir/psql -h $host -p $port -U $user -d $db -c "grant select on \"_${CLUSTER1}\".${table} to ${weakuser};" > /dev/null 2> /dev/null
+      done
+      for table in `echo $RWTBLS`; do
+        $pgbindir/psql -h $host -p $port -U $user -d $db -c "grant all on \"_${CLUSTER1}\".${table} to ${weakuser};" > /dev/null 2> /dev/null
+      done
+    fi
+    if [ ${node} -ge ${NUMNODES} ]; then
+       break;
+    else
+       node=$((${node} + 1))
+    fi   
+  done    
+}
+
 drop_databases()
 {
-	alias=1
+	node=1
 	status "dropping database"
 	while : ; do
-	  eval db=\$DB${alias}
-	  eval host=\$HOST${alias}
-	  eval user=\$USER${alias}
-	  eval pgbindir=\$PGBINDIR${alias}
-	  eval port=\$PORT${alias}
+	  eval db=\$DB${node}
+	  eval host=\$HOST${node}
+	  eval user=\$USER${node}
+	  eval pgbindir=\$PGBINDIR${node}
+	  eval port=\$PORT${node}
 
 	  status "${db}"
 
 	  if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
-	    $pgbindir/dropdb -U $user -h $host -p $port $db 1> ${mktmp}/dropdb.${alias} 2> ${mktmp}/dropdb.${alias}
-	    if [ ${alias} -ge ${NUMNODES} ]; then
+	    $pgbindir/dropdb -U $user -h $host -p $port $db 1> ${mktmp}/dropdb.${node} 2> ${mktmp}/dropdb.${node}
+	    if [ ${node} -ge ${NUMNODES} ]; then
               break;
             else
-              alias=$((${alias} + 1))
+              node=$((${node} + 1))
             fi
 	  else
             break;
@@ -447,9 +484,41 @@ launch_poll()
   esac
 }
 
+build_slonconf()
+{
+    node=$1
+    conninfo=$2
+    eval archive=\$ARCHIVE${node}
+    CONFFILE=$mktmp/slon-conf.${node}
+    echo "log_level=2" > ${CONFFILE}
+    echo "vac_frequency=2" >> ${CONFFILE}
+    echo "cleanup_interval=\"30 seconds\"" >> ${CONFFILE}
+    echo "sync_interval=2010" >> ${CONFFILE}
+    echo "sync_interval_timeout=15000" >> ${CONFFILE}
+    echo "sync_group_maxsize=8" >> ${CONFFILE}
+    echo "sync_max_rowsize=4096" >> ${CONFFILE}
+    echo "sync_max_largemem=1048576" >> ${CONFFILE}
+    echo "syslog=1" >> ${CONFFILE}
+    echo "log_timestamp=true" >> ${CONFFILE}
+    echo "log_timestamp_format='%Y-%m-%d %H:%M:%S %Z'" >> ${CONFFILE}
+    echo "pid_file='${mktmp}/slon-pid.${node}'" >> ${CONFFILE}
+    echo "syslog_facility=LOCAL0" >> ${CONFFILE}
+    echo "syslog_ident=slon-${cluster}-${node}" >> ${CONFFILE}
+    echo "cluster_name='${cluster}'" >> ${CONFFILE}
+    echo "conn_info='${conninfo}'" >> ${CONFFILE}
+    echo "desired_sync_time=60000" >> ${CONFFILE}
+    echo "sql_on_connection=\"SET log_min_duration_statement to '1000';\"" >> ${CONFFILE}
+    echo "lag_interval=\"2 seconds\"" >> ${CONFFILE}
+    if [ "x${archive}" = "xtrue" ]; then
+	status "slonconf configures archive logging for node ${node}"
+	echo "archive_dir='${mktmp}/archive_logs_${node}'" >> ${CONFFILE}
+	eval pgbindir=\$PGBINDIR${node}
+    fi
+}
+
 launch_slon()
 {
-  alias=1
+  node=1
   originnode=${ORIGINNODE:-"1"}
   eval odb=\$DB${originnode}
   eval ohost=\$HOST${originnode}
@@ -459,70 +528,78 @@ launch_slon()
   eval cluster=\$CLUSTER1
 
   conninfo="dbname=${odb} host=${ohost} user=${ouser} port=${oport}"
-  status "launching originnode : $opgbindir/slon -s500 -g10 -d2 $cluster \"$conninfo\""
-  $opgbindir/slon -s500 -g10 -d2 $cluster "$conninfo" 1> $mktmp/slon_log.${originnode} 2> $mktmp/slon_log.${originnode} &
-  tmppid=$!
-  tmpppid=$$
+  build_slonconf ${originnode} "${conninfo}"
+  slonparms=" -f ${mktmp}/slon-conf.${originnode} "
+  status "launching originnode - PGBINDIR=${PGBINDIR}"
+  SLON_BUILD=${PGBINDIR} SLON_CONF=${mktmp}/slon-conf.${originnode} SLON_LOG=$mktmp/slon_log.${originnode} $SLTOOLDIR/start_slon.sh start
+
   sleep 1
-  foo=$(_check_pid slon ${tmppid} ${tmpppid})
-          
-  eval slon${originnode}_pid=${foo}
-  if [ -z "${foo}" -o "${tmppid}" != "${foo}" ]; then
+  SLPID=`SLON_BUILD=$PGBINDIR SLON_CONF=${mktmp}/slon-conf.${originnode} $SLTOOLDIR/start_slon.sh status | grep "Slon running as PID:"`
+  if [ -z "${SLPID}" ]; then
+      echo "SLPID: ${SLPID}"
     warn 3 "Failed to launch slon on node ${originnode} check $mktmp/slon_log.${originnode} for details"
   fi
 
   sleep 10
 
+  node=1
   while : ; do
-    if [ ${alias} -ne ${originnode} ]; then
+      eval archive=\$ARCHIVE${node}
+      if [ "x${archive}" = "xtrue" ]; then
+	  ARCHIVENODE=${node}
+      fi
+      if [ ${node} -ge ${NUMNODES} ]; then
+	  break;
+      else
+	  node=$((${node} + 1))
+      fi
+  done
+  status "Archive node: ${ARCHIVENODE}"
 
-      eval db=\$DB${alias}
-      eval host=\$HOST${alias}
-      eval user=\$USER${alias}
-      eval pgbindir=\$PGBINDIR${alias}
-      eval port=\$PORT${alias}
+  node=1
+  while : ; do
+    if [ ${node} -ne ${originnode} ]; then
+
+      eval db=\$DB${node}
+      eval host=\$HOST${node}
+      eval user=\$USER${node}
+      eval pgbindir=\$PGBINDIR${node}
+      eval port=\$PORT${node}
       eval cluster=\$CLUSTER1
-      eval archive=\$ARCHIVE{alias}
-      eval logship=\$LOGSHIP${alias}
+      eval archive=\$ARCHIVE${node}
 
       if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
         unset conninfo
-
         unset tmppid
         unset tmpppid
-        eval slon${alias}_pid=
+        eval slon${node}_pid=
+	status "Considering node ${node}"
 
-        if [ "x${archive}" != "xtrue" ]; then
-          status "Creating log shipping directory - $mktmp/archive_logs_${alias}"
-          mkdir -p $mktmp/archive_logs_${alias}
-          archiveparm="-a ${mktmp}/archive_logs_${alias}"
-       fi
+        if [ "x${archive}" = "xtrue" ]; then
+          status "Creating log shipping directory - $mktmp/archive_logs_${node}"
+          mkdir -p $mktmp/archive_logs_${node}
+          archiveparm="-a ${mktmp}/archive_logs_${node}"
+	else
+	  archiveparm=""
+        fi
         conninfo="dbname=${db} host=${host} user=${user} port=${port}"
 
-        if [ "x${logship}" == "xtrue" ]; then
-          status "do not launch slon for node ${alias} - it receives data via log shipping"
-        else
-          status "launching: $pgbindir/slon -s500 -g10 -d2 ${archiveparm} $cluster \"$conninfo\""
-
-          $pgbindir/slon -s500 -g10 -d2 ${archiveparm} $cluster "$conninfo" 1>> $mktmp/slon_log.${alias} 2>&1 &
-          tmppid=$!
-          tmpppid=$$
-          sleep 1
-
-          foo=$(_check_pid slon ${tmppid} ${tmpppid})
-
-
-          eval slon${alias}_pid=${foo}
-          if [ -z "${foo}" -o "${tmppid}" != "${foo}" ]; then
-            warn 3 "Failed to launch slon on node ${alias} check $mktmp/slon_log.${alias} for details"
-          fi
-        fi
+	build_slonconf ${node} "${conninfo}"
+	status "launching slon"
+	SLON_BUILD=$PGBINDIR SLON_CONF=${CONFFILE} SLON_LOG=$mktmp/slon_log.${node} $SLTOOLDIR/start_slon.sh start
+	sleep 1
+	
+	SLPID=`SLON_BUILD=${PGBINDIR} SLON_CONF=${mktmp}/slon-conf.${node} $SLTOOLDIR/start_slon.sh status | grep "Slon running as PID:"`
+	if [ -z "${SLPID}" ]; then
+	    echo "SLPID: ${SLPID}"
+	    warn 3 "Failed to launch slon on node ${node} check $mktmp/slon_log.${node} for details"
+	fi
       fi
     fi
-    if [ ${alias} -ge ${NUMNODES} ]; then
+    if [ ${node} -ge ${NUMNODES} ]; then
       break;
     else
-      alias=$((${alias} + 1))
+      node=$((${node} + 1))
     fi
   done
 }
@@ -536,16 +613,16 @@ poll_cluster()
   eval opgbindir=\$PGBINDIR${originnode}
   eval oport=\$PORT${originnode}
 
-  alias=1
+  node=1
   while : ; do
-    eval cluster=\$CLUSTER${alias}
+    eval cluster=\$CLUSTER${node}
     if [ -n "${cluster}" ]; then
       SQL="SELECT * FROM \"_${cluster}\".sl_status"
       ${opgbindir}/psql -c "${SQL}" -h ${ohost} -p ${oport} ${odb} ${ouser}
-      if [ ${alias} -ge ${NUMCLUSTERS} ]; then
+      if [ ${node} -ge ${NUMCLUSTERS} ]; then
         break;
       else
-        alias=expr ${alias} + 1
+        node=`expr ${node} + 1`
       fi
     else
       break;
@@ -553,19 +630,29 @@ poll_cluster()
   done
 }      
 
-wait_for_catchup()
+# wait_for_catchup()
+# {
+#   node=1
+#   status "waiting for nodes to catch up"
+
+#   poll_cluster
+
+#   sleep 20
+
+#   poll_cluster
+#   status "done"
+# }
+
+wait_for_catchup ()
 {
-  alias=1
-  status "waiting for nodes to catchup"
-
-  poll_cluster
-
-  sleep 20
-
-  poll_cluster
-  status "done"
+    eval onode=${ORIGINNODE:-"1"}
+    status "submit SYNC to node ${onode}, wait for event to propagate to all nodes..."
+    echo "include <${mktmp}/slonik.preamble>;" > $mktmp/wait-for-propagation.slonik
+    echo "sync (ID=${onode});" >> $mktmp/wait-for-propagation.slonik
+    echo "wait for event (origin=${onode},confirmed=ALL,wait on=${onode});" >> $mktmp/wait-for-propagation.slonik
+    $pgbindir/slonik < $mktmp/wait-for-propagation.slonik > $mktmp/wait-for-propagation.log 2>&1
+    status "...event propagated to all nodes"
 }
-
 diff_db()
 {
   originnode=${ORIGINNODE:-"1"}
@@ -583,53 +670,59 @@ diff_db()
     ${opgbindir}/psql -c "${SQL}" -h ${ohost} -p ${oport} ${odb} ${ouser} >> ${mktmp}/db_${originnode}.dmp
   done
   status "done"
-  alias=1
+  node=1
   while : ; do
-    eval db=\$DB${alias}
-    eval host=\$HOST${alias}
-    eval user=\$USER${alias}
-    eval port=\$PORT${alias}
-    eval pgbindir=\$PGBINDIR${alias}
+    eval db=\$DB${node}
+    eval host=\$HOST${node}
+    eval user=\$USER${node}
+    eval port=\$PORT${node}
+    eval pgbindir=\$PGBINDIR${node}
 
     if [ -n "${db}" -a "${host}" -a "${user}" -a "${port}" ]; then
-      if [ ${alias} -ne ${originnode} ]; then
-        if [ -f ${mktmp}/db_${alias}.dmp ]; then
-          err 3 "${mktmp}/db_${alias}.dmp exists but should not"
+      if [ ${node} -ne ${originnode} ]; then
+        if [ -f ${mktmp}/db_${node}.dmp ]; then
+          err 3 "${mktmp}/db_${node}.dmp exists but should not"
         fi
-	status "getting data from node ${alias} for diffing against origin"
+	status "getting data from node ${node} for diffing against origin"
         cat ${testname}/schema.diff | while read SQL; do
-          ${pgbindir}/psql -h ${host} -c "${SQL}" -p ${port} ${db} ${user} >> ${mktmp}/db_${alias}.dmp
+          ${pgbindir}/psql -h ${host} -c "${SQL}" -p ${port} ${db} ${user} >> ${mktmp}/db_${node}.dmp
         done
 	status "comparing"
-	diff ${mktmp}/db_${originnode}.dmp ${mktmp}/db_${alias}.dmp > ${mktmp}/db_diff.${alias}
+	diff ${mktmp}/db_${originnode}.dmp ${mktmp}/db_${node}.dmp > ${mktmp}/db_diff.${node}
         if [ $? -ne 0 ]; then
-          warn 3 "${mktmp}/db_${originnode}.dmp ${mktmp}/db_${alias}.dmp differ, see ${mktmp}/db_diff.${alias} for details"
+          warn 3 "${mktmp}/db_${originnode}.dmp ${mktmp}/db_${node}.dmp differ, see ${mktmp}/db_diff.${node} for details"
 	else
-	  echo "subscriber node ${alias} is the same as origin node ${originnode}"
+	  echo "subscriber node ${node} is the same as origin node ${originnode}"
         fi
 	status "done"
       fi
     else
       err 3 "Not Found ${db} -a ${host} ${user}"
     fi
-    if [ ${alias} -ge ${NUMNODES} ]; then
+    if [ ${node} -ge ${NUMNODES} ]; then
       break;
     else
-      alias=$((${alias} + 1))
+      node=$((${node} + 1))
     fi
   done
 }
 
+TMPDIR=${TMPDIR:-"/tmp"}    # Consider either a default value, or /tmp
+if [ -d ${TMPDIR} ]; then
+    T_TMPDIR=${TMPDIR}
+else
+    for i in /tmp /usr/tmp /var/tmp; do
+	if [ -d ${i} ]; then
+	    TMPDIR=$i
+	    break
+	fi
+    done
+fi
+
 case `uname` in
   SunOS|AIX|MINGW32*)
-        for i in /tmp /usr/tmp /var/tmp; do
-      if [ -d $i ]; then
-        tmpdir=$i
-        break
-      fi
-    done
-    if [ -z $tmpdir ]; then
-       err 3 "unable to create tmp dir"
+    if [ -z $TMPDIR ]; then
+       err 3 "unable to create temp dir"
        exit
     fi
 
@@ -637,18 +730,18 @@ case `uname` in
     rstring=$(random_az 8)
     rstring=slony-regress.$rstring
     
-    mkdir $tmpdir/$rstring
+    mkdir ${TMPDIR}/$rstring
     retcode=$?
     if [ $retcode -ne 0 ]; then
       err $retcode "unable to create temp dir"
     else
-      mktmp=$tmpdir/$rstring
+      mktmp=${TMPDIR}/$rstring
     fi
   ;;
   Linux)
     mktmp=`mktemp -d -t slony-regress.XXXXXX`
     if [ $MY_MKTEMP_IS_DECREPIT ] ; then
-        mktmp=`mktemp -d /tmp/slony-regress.XXXXXX`
+        mktmp=`mktemp -d ${TMPDIR}/slony-regress.XXXXXX`
     fi
     if [ ! -d $mktmp ]; then
       err 3 "mktemp failed"
@@ -685,6 +778,10 @@ store_node
 do_ik
 status "done"
 
+status "Granting weak access on Slony-I schema"
+generate_weak_slony_grants
+status "done"
+
 status "storing paths"
 init_preamble
 store_path
@@ -704,12 +801,12 @@ if [ ! -z ${poll_pid} ]; then
 fi
 
 diff_db
-
 stop_processes
 
 status "waiting for slons to die"
 sleep 5
 status "done"
 
+gen_testinfo
 drop_databases
 cleanup
