@@ -1,5 +1,5 @@
 #!@@PERL@@
-# 
+# $Id: slon_watchdog.pl,v 1.15.2.2 2009-08-17 17:09:59 devrim Exp $
 # Author: Christopher Browne
 # Copyright 2004-2009 Afilias Canada
 
@@ -27,7 +27,7 @@ if ($SHOW_USAGE or scalar(@ARGV) != 2) {
   die $USAGE;
 }
 
-require '@@PGLIBDIR@@/slon-tools.pm';
+require '@@PERLSHAREDIR@@/slon-tools.pm';
 require $CONFIG_FILE;
 
 $node = $ARGV[0];
@@ -43,12 +43,17 @@ while (1) {
   $pid = get_pid($node);
   if (!($pid)) {
     my ($dsn, $dbname) = ($DSN[$nodenum], $DBNAME[$nodenum]);
-    open (SLONLOG, ">>$LOGDIR/slon-$dbname-$node.err");
+    my ($logfile) = "$LOGDIR/slon-$dbname-$node.err";
+    open (SLONLOG, ">>$logfile");
     print SLONLOG "WATCHDOG: No Slon is running for node $node!\n";
     print SLONLOG "WATCHDOG: You ought to check the postmaster and slon for evidence of a crash!\n";
     print SLONLOG "WATCHDOG: I'm going to restart slon for $node...\n";
     # First, restart the node using slonik
-    system "@@TOOLSBIN@@/slonik_restart_node $node | @@SLONBINDIR@@/slonik";
+    if ($CONFIG_FILE ne "") {
+      system "(@@TOOLSBIN@@/slonik_restart_node --config=${CONFIG_FILE} $node | @@SLONBINDIR@@/slonik) >> $logfile 2>> $logfile";
+    } else {
+      system "(@@TOOLSBIN@@/slonik_restart_node $node | @@SLONBINDIR@@/slonik) >> $logfile 2>> $logfile";
+    }
     # Next, restart the slon process to service the node
     start_slon($nodenum);
     $pid = get_pid($node);
