@@ -1,5 +1,5 @@
 #!@@PERL@@
-# 
+# $Id: slonik_merge_sets.pl,v 1.3.2.2 2009-08-17 17:09:59 devrim Exp $
 # Author: Christopher Browne
 # Copyright 2004-2009 Afilias Canada
 
@@ -14,9 +14,10 @@ GetOptions("config=s" => \$CONFIG_FILE,
 	   "help"     => \$SHOW_USAGE);
 
 my $USAGE =
-"Usage: merge_sets [--config file] node# set# set#
+"Usage: merge_sets [--config file] node# targetset# srcset#
 
     Merges the contents of the second set into the first one.
+    When completed, the second set no longer exists.
 
 ";
 
@@ -25,7 +26,7 @@ if ($SHOW_USAGE) {
   exit 0;
 }
 
-require '@@PGLIBDIR@@/slon-tools.pm';
+require '@@PERLSHAREDIR@@/slon-tools.pm';
 require $CONFIG_FILE;
 
 my ($node, $set1, $set2) = @ARGV;
@@ -59,9 +60,9 @@ $slonik .= genheader();
 $slonik .= "  try {\n";
 $slonik .= "    merge set (id = $set1, add id = $set2, origin = $node);\n";
 $slonik .= "  } on error {\n";
-$slonik .= "    echo 'Failure to merge sets $set1 and $set2 with origin $node';\n";
+$slonik .= "    echo 'Failure to merge set $set2 into $set1 with origin $node';\n";
 $slonik .= "    exit 1;\n";
 $slonik .= "  }\n";
-$slonik .= "  echo 'Replication set $set2 merged in with $set1 on origin $node';\n";
+$slonik .= "  echo 'Replication set $set2 merged into $set1 on origin $node. Set $set2 no longer exists.';\n";
 
 run_slonik_script($slonik, 'MERGE SET');
