@@ -1,5 +1,5 @@
 #!@@PERL@@
-# 
+# $Id: slonik_subscribe_set.pl,v 1.3.2.1 2009-08-17 17:09:59 devrim Exp $
 # Author: Christopher Browne
 # Copyright 2004-2009 Afilias Canada
 
@@ -36,8 +36,14 @@ if ($node =~ /^(?:node)?(\d+)$/) {
   die $USAGE;
 }
 
-die $USAGE unless $set;
-$set = get_set($set) or die "Non-existent set specified.\n";
+if ($set =~ /^(?:set)?(\d+)$/) {
+  $set = $1;
+} else {
+  print "Need to specify set!\n";
+  die $USAGE;
+}
+
+get_set($set) or die "Non-existent set specified.\n";
 
 my $slonik = '';
 
