@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# 
+# $Id: test_slony_state.pl,v 1.8.2.3 2010-02-10 22:50:51 cbbrowne Exp $
 # Christopher Browne
 # Copyright 2005-2009
 # PostgreSQL Global Development Group
@@ -8,6 +8,9 @@
 # submits a number of queries to test the state of the nodes in the
 # cluster.
 
+# Note: This uses the legacy "Pg" module; it is likely that the other
+# version of this script that uses DBI is preferable to use.
+
 use Pg;
 use Getopt::Long;
 #use strict;
@@ -252,7 +255,7 @@ Could listen paths be missing so that confirmations are not propagating?
   print "\n";
 
   print "\n------------------------------------------------------------------------------\n";
-  print "\nListing of old open connections\n";
+  printf "\nListing of old open connections on node %d\n", $node;
   printf "%15s %15s %15s %12s %20s\n", "Database", "PID", "User", "Query Age", "Query";
   print "================================================================================\n";
 
